@@ -40,18 +40,25 @@
 
 #define THUMBNAIL_WIDTH 100
 
-/* Amount of time we devote to each iteration of the idle, in microseconds */
-#define IDLE_WORK_LENGTH 5000
+/* The IconView doesn't scale nearly as well as the TreeView, so we arbitrarily
+ * limit its use */
+#define MAX_ICON_VIEW_PAGE_COUNT 1500
+
 
 struct _EvSidebarThumbnailsPrivate {
+	GtkWidget *swindow;
+	GtkWidget *icon_view;
 	GtkWidget *tree_view;
 	GtkAdjustment *vadjustment;
 	GtkListStore *list_store;
 	GdkPixbuf *loading_icon;
 	EvDocument *document;
+	EvPageCache *page_cache;
 
 	gint n_pages, pages_done;
 
+	int rotation;
+
 	/* Visible pages */
 	gint start_page, end_page;
 };
@@ -64,6 +71,11 @@ enum {
 	NUM_COLUMNS
 };
 
+enum {
+	PROP_0,
+	PROP_WIDGET,
+};
+
 static void         ev_sidebar_thumbnails_clear_model      (EvSidebarThumbnails *sidebar);
 static gboolean     ev_sidebar_thumbnails_support_document (EvSidebarPage       *sidebar_page,
 							    EvDocument          *document);
@@ -90,12 +102,42 @@ ev_sidebar_thumbnails_dispose (GObject *object)
 {
 	EvSidebarThumbnails *sidebar_thumbnails = EV_SIDEBAR_THUMBNAILS (object);
 	
-	ev_sidebar_thumbnails_clear_model (sidebar_thumbnails);
-	g_object_unref (sidebar_thumbnails->priv->loading_icon);
-
+	if (sidebar_thumbnails->priv->loading_icon) {
+		g_object_unref (sidebar_thumbnails->priv->loading_icon);
+		sidebar_thumbnails->priv->loading_icon = NULL;
+	}
+	
+	if (sidebar_thumbnails->priv->list_store) {
+		ev_sidebar_thumbnails_clear_model (sidebar_thumbnails);
+		g_object_unref (sidebar_thumbnails->priv->list_store);
+		sidebar_thumbnails->priv->list_store = NULL;
+	}
+	
 	G_OBJECT_CLASS (ev_sidebar_thumbnails_parent_class)->dispose (object);
 }
 
+static void
+ev_sidebar_thumbnails_get_property (GObject    *object,
+				    guint       prop_id,
+				    GValue     *value,
+				    GParamSpec *pspec)
+{
+	EvSidebarThumbnails *sidebar = EV_SIDEBAR_THUMBNAILS (object);
+
+	switch (prop_id)
+	{
+	case PROP_WIDGET:
+		if (sidebar->priv->tree_view)
+			g_value_set_object (value, sidebar->priv->tree_view);
+		else
+			g_value_set_object (value, sidebar->priv->icon_view);
+		break;
+	default:
+		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+		break;
+	}
+}
+
 static void
 ev_sidebar_thumbnails_class_init (EvSidebarThumbnailsClass *ev_sidebar_thumbnails_class)
 {
@@ -106,6 +148,11 @@ ev_sidebar_thumbnails_class_init (EvSidebarThumbnailsClass *ev_sidebar_thumbnail
 	gtk_object_class = GTK_OBJECT_CLASS (ev_sidebar_thumbnails_class);
 
 	g_object_class->dispose = ev_sidebar_thumbnails_dispose;
+	g_object_class->get_property = ev_sidebar_thumbnails_get_property;
+
+	g_object_class_override_property (g_object_class,
+					  PROP_WIDGET,
+					  "main-widget");
 
 	g_type_class_add_private (g_object_class, sizeof (EvSidebarThumbnailsPrivate));
 }
@@ -125,13 +172,11 @@ clear_range (EvSidebarThumbnails *sidebar_thumbnails,
 	     gint                 start_page,
 	     gint                 end_page)
 {
-	EvSidebarThumbnailsPrivate *priv;
+	EvSidebarThumbnailsPrivate *priv = sidebar_thumbnails->priv;
 	GtkTreePath *path;
 	GtkTreeIter iter;
 	gboolean result;
 
-	priv = sidebar_thumbnails->priv = EV_SIDEBAR_THUMBNAILS_GET_PRIVATE (sidebar_thumbnails);
-
 	g_assert (start_page <= end_page);
 
 	path = gtk_tree_path_new_from_indices (start_page, -1);
@@ -165,14 +210,12 @@ add_range (EvSidebarThumbnails *sidebar_thumbnails,
 	   gint                 start_page,
 	   gint                 end_page)
 {
-	EvSidebarThumbnailsPrivate *priv;
+	EvSidebarThumbnailsPrivate *priv = sidebar_thumbnails->priv;
 	GtkTreePath *path;
 	GtkTreeIter iter;
 	gboolean result;
 	gint page = start_page;
 
-	priv = sidebar_thumbnails->priv = EV_SIDEBAR_THUMBNAILS_GET_PRIVATE (sidebar_thumbnails);
-
 	g_assert (start_page <= end_page);
 
 	path = gtk_tree_path_new_from_indices (start_page, -1);
@@ -188,7 +231,8 @@ add_range (EvSidebarThumbnails *sidebar_thumbnails,
 				    -1);
 
 		if (job == NULL && !thumbnail_set) {
-			job = (EvJobThumbnail *)ev_job_thumbnail_new (priv->document, page, THUMBNAIL_WIDTH);
+			/* FIXME: Need rotation */
+			job = (EvJobThumbnail *)ev_job_thumbnail_new (priv->document, page, priv->rotation, THUMBNAIL_WIDTH);
 			ev_job_queue_add_job (EV_JOB (job), EV_JOB_PRIORITY_HIGH);
 			g_object_set_data_full (G_OBJECT (job), "tree_iter",
 						gtk_tree_iter_copy (&iter),
@@ -214,11 +258,9 @@ update_visible_range (EvSidebarThumbnails *sidebar_thumbnails,
 		      gint                 start_page,
 		      gint                 end_page)
 {
-	EvSidebarThumbnailsPrivate *priv;
+	EvSidebarThumbnailsPrivate *priv = sidebar_thumbnails->priv;
 	int old_start_page, old_end_page;
 
-	priv = sidebar_thumbnails->priv = EV_SIDEBAR_THUMBNAILS_GET_PRIVATE (sidebar_thumbnails);
-
 	old_start_page = priv->start_page;
 	old_end_page = priv->end_page;
 
@@ -242,29 +284,37 @@ update_visible_range (EvSidebarThumbnails *sidebar_thumbnails,
 static void
 adjustment_changed_cb (EvSidebarThumbnails *sidebar_thumbnails)
 {
-	EvSidebarThumbnailsPrivate *priv;
-	GtkTreePath *path;
-	GtkTreePath *path2;
+	EvSidebarThumbnailsPrivate *priv = sidebar_thumbnails->priv;
+	GtkTreePath *path = NULL;
+	GtkTreePath *path2 = NULL;
 	gint wy1;
 	gint wy2;
 
-	priv = sidebar_thumbnails->priv = EV_SIDEBAR_THUMBNAILS_GET_PRIVATE (sidebar_thumbnails);
+	if (priv->tree_view) {
+		if (! GTK_WIDGET_REALIZED (priv->tree_view))
+			return;
 
-	if (! GTK_WIDGET_REALIZED (priv->tree_view))
+		gtk_tree_view_tree_to_widget_coords (GTK_TREE_VIEW (priv->tree_view),
+						     0, (int) priv->vadjustment->value,
+						     NULL, &wy1);
+		gtk_tree_view_tree_to_widget_coords (GTK_TREE_VIEW (priv->tree_view),
+						     0, (int) (priv->vadjustment->value + priv->vadjustment->page_size),
+						     NULL, &wy2);
+		gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (priv->tree_view),
+					       1, wy1 + 1, &path,
+					       NULL, NULL, NULL);
+		gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (priv->tree_view),
+					       1, wy2 -1, &path2,
+					       NULL, NULL, NULL);
+	} else if (priv->icon_view) {
+		if (! GTK_WIDGET_REALIZED (priv->icon_view))
+			return;
+		if (! gtk_icon_view_get_visible_range (GTK_ICON_VIEW (priv->icon_view), &path, &path2))
+			return;
+	} else {
 		return;
+	}
 
-	gtk_tree_view_tree_to_widget_coords (GTK_TREE_VIEW (priv->tree_view),
-					     0, (int) priv->vadjustment->value,
-					     NULL, &wy1);
-	gtk_tree_view_tree_to_widget_coords (GTK_TREE_VIEW (priv->tree_view),
-					     0, (int) (priv->vadjustment->value + priv->vadjustment->page_size),
-					     NULL, &wy2);
-	gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (priv->tree_view),
-				       1, wy1 + 1, &path,
-				       NULL, NULL, NULL);
-	gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (priv->tree_view),
-				       1, wy2 -1, &path2,
-				       NULL, NULL, NULL);
 	if (path == NULL)
 		path = gtk_tree_path_new_first ();
 	if (path2 == NULL)
@@ -278,18 +328,84 @@ adjustment_changed_cb (EvSidebarThumbnails *sidebar_thumbnails)
 	gtk_tree_path_free (path2);
 }
 
+static void
+ev_sidebar_thumbnails_fill_model (EvSidebarThumbnails *sidebar_thumbnails)
+{
+	EvSidebarThumbnailsPrivate *priv = sidebar_thumbnails->priv;
+	GtkTreeIter iter;
+	int i;
+
+	for (i = 0; i < sidebar_thumbnails->priv->n_pages; i++) {
+		gchar *page_label;
+		gchar *page_string;
+
+		page_label = ev_page_cache_get_page_label (priv->page_cache, i);
+		page_string = g_markup_printf_escaped ("<i>%s</i>", page_label);
+
+		gtk_list_store_append (priv->list_store, &iter);
+		gtk_list_store_set (priv->list_store, &iter,
+				    COLUMN_PAGE_STRING, page_string,
+				    COLUMN_PIXBUF, priv->loading_icon,
+				    COLUMN_THUMBNAIL_SET, FALSE,
+				    -1);
+		g_free (page_label);
+		g_free (page_string);
+	}
+}
+
+
+static void
+ev_sidebar_thumbnails_set_loading_icon (EvSidebarThumbnails *sidebar_thumbnails)
+{
+	gint width = THUMBNAIL_WIDTH;
+	gint height = THUMBNAIL_WIDTH;
+
+	if (sidebar_thumbnails->priv->loading_icon)
+		g_object_unref (sidebar_thumbnails->priv->loading_icon);
+
+	if (sidebar_thumbnails->priv->document) {
+		/* We get the dimensions of the first doc so that we can make a blank
+		 * icon.  */
+		ev_document_doc_mutex_lock ();
+		ev_document_thumbnails_get_dimensions (EV_DOCUMENT_THUMBNAILS (sidebar_thumbnails->priv->document),
+						       0, THUMBNAIL_WIDTH, &width, &height);
+		ev_document_doc_mutex_unlock ();
+		sidebar_thumbnails->priv->loading_icon =
+			ev_document_misc_get_thumbnail_frame (width, height, sidebar_thumbnails->priv->rotation, NULL);
+	} else {
+		sidebar_thumbnails->priv->loading_icon = NULL;
+	}
+
+}
+void
+ev_sidebar_thumbnails_refresh (EvSidebarThumbnails *sidebar_thumbnails,
+
+			       int                  rotation)
+{
+	sidebar_thumbnails->priv->rotation = rotation;
+	ev_sidebar_thumbnails_set_loading_icon (sidebar_thumbnails);
+
+	if (sidebar_thumbnails->priv->document == NULL)
+		return;
+
+	ev_sidebar_thumbnails_clear_model (sidebar_thumbnails);
+	ev_sidebar_thumbnails_fill_model (sidebar_thumbnails);
+
+	/* Trigger a redraw */
+	sidebar_thumbnails->priv->start_page = 0;
+	sidebar_thumbnails->priv->end_page = 0;
+	adjustment_changed_cb (sidebar_thumbnails);
+}
+
 static void
 ev_sidebar_tree_selection_changed (GtkTreeSelection *selection,
 				   EvSidebarThumbnails *ev_sidebar_thumbnails)
 {
-	EvSidebarThumbnailsPrivate *priv;
-	EvPageCache *page_cache;
+	EvSidebarThumbnailsPrivate *priv = ev_sidebar_thumbnails->priv;
 	GtkTreePath *path;
 	GtkTreeIter iter;
 	int page;
 
-	priv = ev_sidebar_thumbnails->priv = EV_SIDEBAR_THUMBNAILS_GET_PRIVATE (ev_sidebar_thumbnails);
-
 	if (!gtk_tree_selection_get_selected (selection, NULL, &iter))
 		return;
 
@@ -298,27 +414,42 @@ ev_sidebar_tree_selection_changed (GtkTreeSelection *selection,
 	page = gtk_tree_path_get_indices (path)[0];
 	gtk_tree_path_free (path);
 
-	page_cache = ev_document_get_page_cache (priv->document);
-	ev_page_cache_set_current_page (page_cache, page);
-}
-
-GtkWidget *
-ev_sidebar_thumbnails_get_treeview (EvSidebarThumbnails *sidebar)
-{
-	return sidebar->priv->tree_view;
+	ev_page_cache_set_current_page (priv->page_cache, page);
 }
 
 static void
-ev_sidebar_thumbnails_init (EvSidebarThumbnails *ev_sidebar_thumbnails)
+ev_sidebar_icon_selection_changed (GtkIconView         *icon_view,
+				   EvSidebarThumbnails *ev_sidebar_thumbnails)
+{
+	EvSidebarThumbnailsPrivate *priv = ev_sidebar_thumbnails->priv;
+	GtkTreePath *path;
+	GList *selected;
+	int page;
+
+	selected = gtk_icon_view_get_selected_items (icon_view);
+	if (selected == NULL)
+		return;
+
+	/* We don't handle or expect multiple selection. */
+	g_assert (selected->next == NULL);
+
+	path = selected->data;
+	page = gtk_tree_path_get_indices (path)[0];
+
+	gtk_tree_path_free (path);
+	g_list_free (selected);
+
+	ev_page_cache_set_current_page (priv->page_cache, page);
+}
+
+static void
+ev_sidebar_init_tree_view (EvSidebarThumbnails *ev_sidebar_thumbnails)
 {
-	GtkWidget *swindow;
 	EvSidebarThumbnailsPrivate *priv;
-	GtkCellRenderer *renderer;
 	GtkTreeSelection *selection;
+	GtkCellRenderer *renderer;
 
-	priv = ev_sidebar_thumbnails->priv = EV_SIDEBAR_THUMBNAILS_GET_PRIVATE (ev_sidebar_thumbnails);
-
-	priv->list_store = gtk_list_store_new (NUM_COLUMNS, G_TYPE_STRING, GDK_TYPE_PIXBUF, G_TYPE_BOOLEAN, EV_TYPE_JOB_THUMBNAIL);
+	priv = ev_sidebar_thumbnails->priv;
 	priv->tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL (priv->list_store));
 
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (priv->tree_view));
@@ -336,26 +467,66 @@ ev_sidebar_thumbnails_init (EvSidebarThumbnails *ev_sidebar_thumbnails)
 	gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (priv->tree_view), -1,
 						     NULL, gtk_cell_renderer_text_new (),
 						     "markup", 0, NULL);
+	gtk_container_add (GTK_CONTAINER (priv->swindow), priv->tree_view);
+	gtk_widget_show (priv->tree_view);
+}
 
-	g_object_unref (priv->list_store);
+static void
+ev_sidebar_init_icon_view (EvSidebarThumbnails *ev_sidebar_thumbnails)
+{
+	EvSidebarThumbnailsPrivate *priv;
 
-	swindow = gtk_scrolled_window_new (NULL, NULL);
-	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (swindow),
+	priv = ev_sidebar_thumbnails->priv;
+
+	priv->icon_view = gtk_icon_view_new_with_model (GTK_TREE_MODEL (priv->list_store));
+	gtk_icon_view_set_markup_column (GTK_ICON_VIEW (priv->icon_view), 0);
+	gtk_icon_view_set_pixbuf_column (GTK_ICON_VIEW (priv->icon_view), 1);
+	g_signal_connect (priv->icon_view, "selection-changed",
+			  G_CALLBACK (ev_sidebar_icon_selection_changed), ev_sidebar_thumbnails);
+
+	gtk_container_add (GTK_CONTAINER (priv->swindow), priv->icon_view);
+	gtk_widget_show (priv->icon_view);
+}
+
+static gboolean
+ev_sidebar_thumbnails_use_icon_view (EvSidebarThumbnails *sidebar_thumbnails)
+{
+	EvSidebarThumbnailsPrivate *priv = sidebar_thumbnails->priv;
+	if (ev_page_cache_get_n_pages (priv->page_cache) > MAX_ICON_VIEW_PAGE_COUNT)
+		return FALSE;
+	return TRUE;
+}
+
+static void
+ev_sidebar_thumbnails_init (EvSidebarThumbnails *ev_sidebar_thumbnails)
+{
+	EvSidebarThumbnailsPrivate *priv;
+
+	priv = ev_sidebar_thumbnails->priv = EV_SIDEBAR_THUMBNAILS_GET_PRIVATE (ev_sidebar_thumbnails);
+
+	priv->list_store = gtk_list_store_new (NUM_COLUMNS,
+					       G_TYPE_STRING,
+					       GDK_TYPE_PIXBUF,
+					       G_TYPE_BOOLEAN,
+					       EV_TYPE_JOB_THUMBNAIL);
+
+	priv->swindow = gtk_scrolled_window_new (NULL, NULL);
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (priv->swindow),
 					GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (swindow),
+	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (priv->swindow),
 					     GTK_SHADOW_IN);
-	priv->vadjustment = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (swindow));
+	priv->vadjustment = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (priv->swindow));
 	g_signal_connect_data (G_OBJECT (priv->vadjustment), "value-changed",
 			       G_CALLBACK (adjustment_changed_cb),
 			       ev_sidebar_thumbnails, NULL,
 			       G_CONNECT_SWAPPED | G_CONNECT_AFTER);
-	g_signal_connect_swapped (G_OBJECT (swindow), "size-allocate",
+	g_signal_connect_swapped (G_OBJECT (priv->swindow), "size-allocate",
 				  G_CALLBACK (adjustment_changed_cb),
 				  ev_sidebar_thumbnails);
-	gtk_container_add (GTK_CONTAINER (swindow), priv->tree_view);
-	gtk_box_pack_start (GTK_BOX (ev_sidebar_thumbnails), swindow, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (ev_sidebar_thumbnails), priv->swindow, TRUE, TRUE, 0);
 
-	gtk_widget_show_all (swindow);
+	/* Put it all together */
+	gtk_widget_show_all (priv->swindow);
 }
 
 static void
@@ -363,12 +534,20 @@ page_changed_cb (EvPageCache         *page_cache,
 		 int                  page,
 		 EvSidebarThumbnails *sidebar)
 {
-	GtkTreeView *tree_view = GTK_TREE_VIEW (sidebar->priv->tree_view);
+	GtkTreeView *tree_view;
 	GtkTreePath *path;
 
 	path = gtk_tree_path_new_from_indices (page, -1);
-	gtk_tree_view_set_cursor (tree_view, path, NULL, FALSE);
-	gtk_tree_view_scroll_to_cell (tree_view, path, NULL, FALSE, 0.0, 0.0);
+
+	if (sidebar->priv->tree_view) {
+		tree_view = GTK_TREE_VIEW (sidebar->priv->tree_view);
+		gtk_tree_view_set_cursor (tree_view, path, NULL, FALSE);
+		gtk_tree_view_scroll_to_cell (tree_view, path, NULL, FALSE, 0.0, 0.0);
+	} else if (sidebar->priv->icon_view) {
+		gtk_icon_view_select_path (GTK_ICON_VIEW (sidebar->priv->icon_view), path);
+		gtk_icon_view_set_cursor (GTK_ICON_VIEW (sidebar->priv->icon_view), path, NULL, FALSE);
+	}
+
 	gtk_tree_path_free (path);
 }
 
@@ -393,55 +572,48 @@ ev_sidebar_thumbnails_set_document (EvSidebarPage	*sidebar_page,
 				    EvDocument          *document)
 {
 	EvSidebarThumbnails *sidebar_thumbnails = EV_SIDEBAR_THUMBNAILS (sidebar_page);
-	gint i, n_pages;
-	GtkTreeIter iter;
-	gint width = THUMBNAIL_WIDTH;
-	gint height = THUMBNAIL_WIDTH;
-	EvPageCache *page_cache;
 
 	EvSidebarThumbnailsPrivate *priv = sidebar_thumbnails->priv;
 
 	g_return_if_fail (EV_IS_DOCUMENT_THUMBNAILS (document));
 
-	page_cache = ev_document_get_page_cache (document);
-	n_pages = ev_page_cache_get_n_pages (page_cache);
-
+	priv->page_cache = ev_page_cache_get (document);
 	priv->document = document;
-	priv->n_pages = n_pages;
+	priv->n_pages = ev_page_cache_get_n_pages (priv->page_cache);
 
-	/* We get the dimensions of the first doc so that we can make a blank
-	 * icon.  */
-	ev_document_doc_mutex_lock ();
-	ev_document_thumbnails_get_dimensions (EV_DOCUMENT_THUMBNAILS (priv->document),
-					       0, THUMBNAIL_WIDTH, &width, &height);
-	ev_document_doc_mutex_unlock ();
-
-	if (priv->loading_icon)
-		g_object_unref (priv->loading_icon);
-	priv->loading_icon = ev_document_misc_get_thumbnail_frame (width, height, NULL);
+	ev_sidebar_thumbnails_set_loading_icon (sidebar_thumbnails);
 
 	ev_sidebar_thumbnails_clear_model (sidebar_thumbnails);
-	for (i = 0; i < n_pages; i++) {
-		gchar *page_label;
-		gchar *page_string;
+	ev_sidebar_thumbnails_fill_model (sidebar_thumbnails);
 
-		page_label = ev_page_cache_get_page_label (page_cache, i);
-		page_string = g_markup_printf_escaped ("<i>%s</i>", page_label);
+	/* Create the view widget, and remove the old one, if needed */
+	if (ev_sidebar_thumbnails_use_icon_view (sidebar_thumbnails)) {
+		if (priv->tree_view) {
+			gtk_container_remove (GTK_CONTAINER (priv->swindow), priv->tree_view);
+			priv->tree_view = NULL;
+		}
 
-		gtk_list_store_append (priv->list_store, &iter);
-		gtk_list_store_set (priv->list_store, &iter,
-				    COLUMN_PAGE_STRING, page_string,
-				    COLUMN_PIXBUF, priv->loading_icon,
-				    COLUMN_THUMBNAIL_SET, FALSE,
-				    -1);
-		g_free (page_label);
-		g_free (page_string);
+		if (! priv->icon_view) {
+			ev_sidebar_init_icon_view (sidebar_thumbnails);
+			g_object_notify (G_OBJECT (sidebar_thumbnails), "main_widget");
+		}
+	} else {
+		if (priv->icon_view) {
+			gtk_container_remove (GTK_CONTAINER (priv->swindow), priv->icon_view);
+			priv->icon_view = NULL;
+		}
+
+		if (! priv->tree_view) {
+			ev_sidebar_init_tree_view (sidebar_thumbnails);
+			g_object_notify (G_OBJECT (sidebar_thumbnails), "main_widget");
+		}
 	}
 
 	/* Connect to the signal and trigger a fake callback */
-	g_signal_connect (page_cache, "page-changed", G_CALLBACK (page_changed_cb), sidebar_thumbnails);
-	page_changed_cb (page_cache, ev_page_cache_get_current_page (page_cache), sidebar_thumbnails);
-
+	g_signal_connect (priv->page_cache, "page-changed", G_CALLBACK (page_changed_cb), sidebar_thumbnails);
+	sidebar_thumbnails->priv->start_page = 0;
+	sidebar_thumbnails->priv->end_page = 0;
+	adjustment_changed_cb (sidebar_thumbnails);
 }
 
 static gboolean
